/*
 * deviced
 *
 * Copyright (c) 2012 - 2013 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <libsyscommon/list.h>

#include "feedback-ids.h"
#include "devices.h"
#include "log.h"

static GList *dev_head;

void add_device(const struct device_ops *dev)
{
	SYS_G_LIST_APPEND(dev_head, (struct device_ops*)dev);
}

//LCOV_EXCL_START System Error
void remove_device(const struct device_ops *dev)
{
	SYS_G_LIST_REMOVE(dev_head, (struct device_ops*)dev);
}
//LCOV_EXCL_STOP

const struct device_ops *find_device(int type)
{
	GList *elem;
	const struct device_ops *dev;

	SYS_G_LIST_FOREACH(dev_head, elem, dev) {
		if (dev->type == type)
			return dev;
	}
	return NULL;
}

void devices_init(void)
{
	GList *elem;
	const struct device_ops *dev;

	SYS_G_LIST_FOREACH(dev_head, elem, dev) {
		_D("[%s] initialize", dev->name);
		if (dev->init)
			dev->init();
	}
}

void devices_exit(void)
{
	GList *elem;
	const struct device_ops *dev;

	SYS_G_LIST_FOREACH(dev_head, elem, dev) {
		_D("[%s] deinitialize", dev->name);
		if (dev->exit)
			dev->exit();
	}
}

//LCOV_EXCL_START
int devices_play(int pattern, bool always)
{
	GList *elem;
	const struct device_ops *dev;
	int ret, prev = -EPERM;

	SYS_G_LIST_FOREACH(dev_head, elem, dev) {
		if (dev->play) {
			ret = dev->play(pattern, always);
			if ((prev < 0 && ret == 0) ||
			    (prev == 0 && ret < 0))
				prev = 0;
			else if ((prev < 0 && ret == -ENOTSUP) ||
					(prev == -ENOTSUP && ret < 0))
				prev = -ENOTSUP;
			else
				prev = ret;
		}
	}

	return prev;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
int devices_play_soundpath(int pattern, const char *soundpath, bool always)
{
	GList *elem;
	const struct device_ops *dev;
	int ret, prev = -EPERM;

	SYS_G_LIST_FOREACH(dev_head, elem, dev) {
		if (dev->type == FEEDBACK_TYPE_SOUND) {
			if (dev->play_path)
				ret = dev->play_path(pattern, soundpath, always);
			else
				continue;	//LCOV_EXCL_LINE
		} else {
			if (dev->play)
				ret = dev->play(pattern, always);
			else
				continue;	//LCOV_EXCL_LINE
		}

		if ((prev < 0 && ret == 0) ||
			(prev == 0 && ret < 0))
			prev = 0;
		else if ((prev < 0 && ret == -ENOTSUP) ||
			(prev == -ENOTSUP && ret < 0))
			prev = -ENOTSUP;
		else
			prev = ret;
	}

	return prev;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
int devices_stop(void)
{
	GList *elem;
	const struct device_ops *dev;
	int ret = -ENOTSUP;

	SYS_G_LIST_FOREACH(dev_head, elem, dev) {
		if (dev->stop)
			ret = dev->stop();
	}

	return ret;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
int devices_stop_by_type(feedback_type_e feedback_type)
{
	GList *elem;
	const struct device_ops *dev;

	SYS_G_LIST_FOREACH(dev_head, elem, dev) {
		if (dev->type == feedback_type)
			return dev->stop();
	}

	return -ENOTSUP;
}
//LCOV_EXCL_STOP
