/*
 * libfeedback
 * Copyright (c) 2023 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>
#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <libsyscommon/ini-parser.h>
#include <libsyscommon/list.h>

#include "log.h"
#include "profiles.h"
#include "sound-theme-manager.h"

#define INITIAL_DEFAULT_SOUND_THEME_ID				-1
#define INITIAL_SOUND_PRIORITY					-1
#define SOUND_CONF_FILE FEEDBACK_SYS_RO_SHARE"/feedback/sound.conf"

static int default_sound_theme_id = INITIAL_DEFAULT_SOUND_THEME_ID;

struct sound_theme_info {
	int id;
	int is_default;
	char *conf_file_path;
};

//LCOV_EXCL_START
static void destroy_hash_table(void *hash_table)
{
	if (hash_table)
		g_hash_table_destroy(g_steal_pointer((GHashTable**)hash_table));
}

static bool is_sound_theme_id_duplicated(int sound_theme_id, void *data)
{
	GHashTable *hash_table_to_check_id_duplicated = (GHashTable *)data;
	gint *hash_key = g_new(gint, 1);
	*hash_key = sound_theme_id;
	if (g_hash_table_contains(hash_table_to_check_id_duplicated, hash_key)) {
		g_free(hash_key);
		return true;
	}

	g_hash_table_insert(hash_table_to_check_id_duplicated, hash_key, NULL);
	return false;
}

static bool is_default_theme_id_set(void)
{
	if (default_sound_theme_id == INITIAL_DEFAULT_SOUND_THEME_ID)
		return false;
	return true;
}

static int parse_sound_theme_property(gpointer data, gpointer user_data)
{
	struct section_property *prop = (struct section_property *) data;
	struct sound_theme_info *sound_theme_elem = (struct sound_theme_info *)user_data;

	if (!prop || !sound_theme_elem)
		return 0;

	if (MATCH(prop->key, "SoundThemeId")) {
		sscanf(prop->value, "%d", (&sound_theme_elem->id));
		if (sound_theme_elem->id < 0)
			return -EPERM;
	} else if (MATCH(prop->key, "SoundThemePath")) {
		int str_len = sizeof(prop->value);
		sound_theme_elem->conf_file_path = strndup(prop->value, str_len);
	} else if (MATCH(prop->key, "SoundThemeDefault")) {
		if (MATCH(prop->value, "yes"))
			sound_theme_elem->is_default = 1;
		else if (MATCH(prop->value, "no"))
			sound_theme_elem->is_default = 0;
		else
			return -EPERM;
	}
	return 0;
}


static int parse_sound_theme_section(const struct parse_result *result, void *data)
{
	struct sound_theme_info sound_theme_elem = {
		.id = 0,
		.is_default = 0,
		.conf_file_path = NULL
	};
	struct section_property *extracted_section_prop = NULL;
	GList *temp_glist = NULL;
	int ret = 0;

	if (!result || !result->props)
		return 0;

	if (!MATCH("SoundTheme", result->section))
		return 0;

	SYS_G_LIST_FOREACH(result->props, temp_glist, extracted_section_prop) {
		if (parse_sound_theme_property(extracted_section_prop, &sound_theme_elem) < 0)
			goto out_parsing_fail;
	}

	if (is_sound_theme_id_duplicated(sound_theme_elem.id, data)) {
		_E("Failed to parse sound conf file, sound theme id is duplicated.");
		goto out_parsing_fail;
	}

	if (sound_theme_elem.is_default) {
		if (is_default_theme_id_set()) {
			_E("Failed to parse sound conf file, default sound theme is duplicated.");
			goto out_parsing_fail;
		}
		default_sound_theme_id = sound_theme_elem.id;
		sound_thememan_set_default_sound_theme_id(default_sound_theme_id);
	}

	ret = sound_thememan_add_sound_theme(sound_theme_elem.id, sound_theme_elem.conf_file_path);
	free(sound_theme_elem.conf_file_path);

	if (ret < 0)
		return ret;

	return 0;

out_parsing_fail:
	_E("Failed to parse sound conf file, please check conf file description and follow the rules");
	free(sound_theme_elem.conf_file_path);

	return -EPERM;
}

static int parse_sound_section(const struct parse_result *result, void *data)
{
	if (!result || !result->props)
		return 0;

	if (MATCH(result->section, "Sound")) {
		if (is_default_theme_id_set()) {
			_E("Failed to parse sound conf file, please check conf file description and follow the rules");
			return -EINVAL;
		}
		default_sound_theme_id = 0;
		sound_thememan_add_sound_theme(default_sound_theme_id, SOUND_CONF_FILE);
		return 0;
	}

	return 0;
}

int sound_parser_init(void)
{
	int ret = 0;
	GHashTable *hash_table_to_check_id_duplicated;

	ret = syscommon_config_parse_by_section(SOUND_CONF_FILE, parse_sound_section, NULL);
	if (ret < 0) {
		_E("Failed to parse sound conf file. [Sound] section parsing failed.");
		return ret;
	}

	if (is_default_theme_id_set())
		return 0;

	hash_table_to_check_id_duplicated = g_hash_table_new_full(g_int_hash, g_int_equal,
									g_free, g_free);
	ret = syscommon_config_parse_by_section(SOUND_CONF_FILE, parse_sound_theme_section, hash_table_to_check_id_duplicated);
	if (ret < 0) {
		_E("Failed to parse sound conf file. [SoundTheme] section parsing failed.");
		goto destroy_hash_table;
	}

	if (!is_default_theme_id_set()) {
		_E("Failed to parse sound conf file. There is no default sound theme id");
		ret = -EPERM;
		goto destroy_hash_table;
	}

destroy_hash_table:
	destroy_hash_table(&hash_table_to_check_id_duplicated);

	return ret;
}

void sound_parser_exit(void)
{
	default_sound_theme_id = INITIAL_DEFAULT_SOUND_THEME_ID;
}

static int parse_sound_property(const struct parse_result *result, void *data)
{
	struct section_property *extracted_section_prop = NULL;
	GList *temp_glist = NULL;
	int pattern = 0;
	GHashTable *sound_config_info;
	struct sound_pattern_data* sound_data = NULL;

	if (!result || !result->props)
		return 0;

	if (!data)
		return -EINVAL;

	if (!MATCH(result->section, "Sound"))
		return 0;

	sound_config_info = (GHashTable*)data;

	SYS_G_LIST_FOREACH(result->props, temp_glist, extracted_section_prop) {
		pattern = profile->get_pattern_enum(extracted_section_prop->key);
		if (pattern < 0 || pattern >= profile->max_pattern)
			return -EINVAL;
		sound_data = (struct sound_pattern_data*)calloc(1, sizeof(struct sound_pattern_data));
		if (!sound_data)
			return -ENOMEM;
		sound_data->path = g_strdup(extracted_section_prop->value);
		sound_data->priority = INITIAL_SOUND_PRIORITY;
		g_hash_table_insert(sound_config_info, GINT_TO_POINTER(pattern),
					(gpointer)sound_data);
	}
	return 0;
}

int sound_parser_get_config_info(const char *file_path, void *config_info)
{
	int ret = 0;
	GHashTable *sound_info;

	if (!file_path || !config_info)
		return -EINVAL;

	sound_info = (GHashTable*)config_info;
	ret = syscommon_config_parse_by_section(file_path, parse_sound_property, sound_info);
	if (ret < 0) {
		_E("Failed to parse sound conf feedback pattern.");
		return -EPERM;
	}

	return 0;
}

static int parse_sound_pattern_priority_property(const struct parse_result *result, void *data)
{
	struct section_property *extracted_section_prop = NULL;
	GList *temp_glist = NULL;
	int pattern = 0;
	GHashTable *sound_config_info = NULL;
	struct sound_pattern_data* sound_data = NULL;

	if (!result || !result->props)
		return 0;

	if (!data)
		return -EINVAL;

	if (!MATCH("SoundPatternPriority", result->section))
		return 0;

	sound_config_info = (GHashTable*)data;

	SYS_G_LIST_FOREACH(result->props, temp_glist, extracted_section_prop) {
		pattern = profile->get_pattern_enum(extracted_section_prop->key);
		if (pattern < 0 || pattern >= profile->max_pattern)
			return -EINVAL;
		sound_data = g_hash_table_lookup(sound_config_info, GINT_TO_POINTER(pattern));
		if (!sound_data) {
			_E("Failed to parse sound pattern priority, [Sound] and [SoundPatternPriority] pattern does not match");
			return -EPERM;
		}
		sound_data->priority = atoi(extracted_section_prop->value);
	}
	return 0;
}

static bool is_sound_section_matching_sound_priority_section(void *data)
{
	GList *value_list = NULL;
	GList *temp_glist = NULL;
	struct sound_pattern_data *sound_data = NULL;
	int number_of_defined_priority_pattern = 0;

	if (!data)
		return -EINVAL;

	value_list = g_hash_table_get_values((GHashTable*)data);

	SYS_G_LIST_FOREACH(value_list, temp_glist, sound_data) {
		if (sound_data && sound_data->priority == INITIAL_SOUND_PRIORITY)
			number_of_defined_priority_pattern++;
	}

	g_list_free(value_list);
	if (number_of_defined_priority_pattern == 0 || number_of_defined_priority_pattern == g_hash_table_size((GHashTable*)data))
		return true;

	_E("Failed to parse sound pattern priority, [Sound] and [SoundPatternPriority] pattern does not match");
	return false;
}

int sound_parser_get_priority_info(const char *file_path, void *priority_info)
{
	int ret = 0;
	GHashTable *sound_info;

	if (!file_path || !priority_info)
		return -EINVAL;

	sound_info = (GHashTable*)priority_info;
	ret = syscommon_config_parse_by_section(file_path, parse_sound_pattern_priority_property, sound_info);
	if (ret < 0)
		return -EPERM;

	if (!is_sound_section_matching_sound_priority_section(sound_info))
		return -EPERM;

	return 0;
}
//LCOV_EXCL_STOP
