/*
 * libfeedback
 * Copyright (c) 2023 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>

#include <libsyscommon/list.h>

#include "feedback-config.h"
#include "profiles.h"
#include "log.h"
#include "sound-parser.h"
#include "sound-theme-manager.h"

#define SOUND_NAME				"Sound"

struct sound_theme_element {
	unsigned int id;
	GHashTable *sound_info;
};

static unsigned int default_sound_theme_id;
static GList *g_sound_theme_list;

//LCOV_EXCL_START
static void cleanup_sound_pattern_data(gpointer data)
{
	struct sound_pattern_data *sound_data = data;
	g_free((char*)sound_data->path);
	free(sound_data);
}

bool sound_thememan_is_sound_theme_id_exist(unsigned int sound_theme_id)
{
	struct sound_theme_element *sound_theme_elem = NULL;
	GList *temp_glist = NULL;

	SYS_G_LIST_FOREACH(g_sound_theme_list, temp_glist, sound_theme_elem) {
		if (sound_theme_elem->id == sound_theme_id) {
			return true;
		}
	}

	return false;
}

void sound_thememan_set_default_sound_theme_id(unsigned int sound_theme_id)
{
	default_sound_theme_id = sound_theme_id;
}

void sound_thememan_get_default_sound_theme_id(unsigned int *sound_theme_id)
{
	*sound_theme_id = default_sound_theme_id;
}

int sound_thememan_get_number_of_theme(unsigned int *number_of_theme)
{
	if (!number_of_theme) {
		_E("Invalid parameter: number_of_theme(NULL)");
		return -EINVAL;
	}

	*number_of_theme = (unsigned int)g_list_length(g_sound_theme_list);
	return 0;
}

int sound_thememan_get_pattern_data(unsigned int sound_theme_id, feedback_pattern_e pattern,
						void *sound_pattern_data)
{
	struct sound_theme_element *sound_theme_elem = NULL;
	GList *temp_glist = NULL;
	struct sound_pattern_data *sound_data = NULL;

	if (!sound_pattern_data)
		return -EINVAL;

	SYS_G_LIST_FOREACH(g_sound_theme_list, temp_glist, sound_theme_elem) {
		if (sound_theme_elem->id == sound_theme_id) {
			sound_data = g_hash_table_lookup(sound_theme_elem->sound_info, GINT_TO_POINTER(pattern));
			if (sound_data) {
				*(struct sound_pattern_data**)sound_pattern_data = sound_data;
				return 0;
			}
		}
	}

	return 0;
}

int sound_thememan_add_sound_theme(unsigned int sound_theme_id, const char* conf_file_path)
{
	struct sound_theme_element *sound_theme_elem = NULL;
	int ret = 0;
	GHashTable *sound_info = g_hash_table_new_full(g_direct_hash, g_direct_equal,
								NULL, cleanup_sound_pattern_data);

	if (!sound_info) {
		_E("Failed to allocate memory for sound_info.");
		return -ENOMEM;
	}

	sound_theme_elem = (struct sound_theme_element*)calloc(1, sizeof(struct sound_theme_element));
	if (!sound_theme_elem) {
		_E("Failed to allocate memory for sound_theme_elem.");
		ret = -ENOMEM;
		goto out_add_sound_theme_fail;
	}

	ret = sound_parser_get_config_info(conf_file_path, sound_info);
	if (ret < 0) {
		_E("Failed to load config file %s", conf_file_path);
		goto out_add_sound_theme_fail;
	}

	ret = sound_parser_get_priority_info(conf_file_path, sound_info);
	if (ret < 0)
		goto out_add_sound_theme_fail;

	sound_theme_elem->id = sound_theme_id;
	sound_theme_elem->sound_info = sound_info;
	g_sound_theme_list = g_list_append(g_sound_theme_list, sound_theme_elem);
	return 0;

out_add_sound_theme_fail:
	if (sound_info)
		g_hash_table_destroy(g_steal_pointer(&sound_info));

	if (sound_theme_elem)
		free(sound_theme_elem);

	return ret;
}

int sound_thememan_get_sound_theme_ids(unsigned int *count_of_theme, unsigned int **sound_theme_ids)
{
	int index = 0;
	unsigned int *temp_sound_theme_ids = NULL;
	struct sound_theme_element* sound_theme_elem = NULL;
	GList *temp_glist = NULL;

	sound_thememan_get_number_of_theme(count_of_theme);

	temp_sound_theme_ids = calloc(*count_of_theme, sizeof(unsigned int));
	if (!temp_sound_theme_ids)
		return -ENOMEM;

	SYS_G_LIST_FOREACH(g_sound_theme_list, temp_glist, sound_theme_elem) {
		temp_sound_theme_ids[index++] = sound_theme_elem->id;
	}

	*sound_theme_ids = g_steal_pointer(&temp_sound_theme_ids);
	return 0;
}

static void cleanup_sound_theme_element(gpointer data)
{
	struct sound_theme_element *sound_theme_elem = NULL;

	if (!data)
		return;

	sound_theme_elem = data;
	if (sound_theme_elem->sound_info)
		g_hash_table_destroy(g_steal_pointer(&sound_theme_elem->sound_info));

	free(sound_theme_elem);
}

int sound_thememan_init(void)
{
	g_sound_theme_list = NULL;
	return 0;
}

void sound_thememan_exit(void)
{
	if (g_sound_theme_list)
		g_list_free_full(g_steal_pointer(&g_sound_theme_list), cleanup_sound_theme_element);
}
//LCOV_EXCL_STOP

