/*
 * libfeedback
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include <vconf.h>
#include <mm_sound_private.h>
#include <sound_manager.h>
#include <wav_player.h>
#include <libsyscommon/ini-parser.h>
#include <libsyscommon/list.h>

#include "feedback-ids.h"
#include "profiles.h"
#include "devices.h"
#include "log.h"
#include "feedback-config.h"
#include "sound-parser.h"
#include "sound-theme-manager.h"

/* Temporary keys */
#ifndef VCONFKEY_SETAPPL_BUTTON_SOUNDS_BOOL
#define VCONFKEY_SETAPPL_BUTTON_SOUNDS_BOOL "db/setting/sound/button_sounds"
#endif

#define SOUND_THEME_ID_UNINITIALIZED 0

static int sndstatus;
static int touch_sndstatus;
static int keytone_sndstatus;
static unsigned int current_theme_id = 0;

static int sound_set_theme_id(unsigned int id_of_theme);

static const char *get_data(feedback_pattern_e pattern)
{
	const char *data;
	struct sound_pattern_data *sound_data = NULL;
	int ret = 0;

	if (pattern <= FEEDBACK_PATTERN_NONE ||
	    pattern >= profile->max_pattern)
		return NULL;

	data = profile->get_path(pattern);
	if (data)
		return data;

	ret = sound_thememan_get_pattern_data(current_theme_id, pattern, &sound_data);
	if (ret < 0)
		return NULL;

	if (!sound_data) {
		_E("Not supported pattern : %d", pattern);
		return NULL;
	}

	return sound_data->path;
}

//LCOV_EXCL_START
static int get_sound_pattern_data(feedback_pattern_e pattern, const char **path, int *priority)
{
	struct sound_pattern_data *sound_data = NULL;
	int ret = 0;

	if (pattern <= FEEDBACK_PATTERN_NONE || pattern >= profile->max_pattern)
		return -EINVAL;

	ret = sound_thememan_get_pattern_data(current_theme_id, pattern, &sound_data);
	if (ret < 0)
		return -EPERM;

	if (!sound_data) {
		_E("Not supported pattern : %d", pattern);
		return -EPERM;
	}

	if (sound_data->path && path)
		*path = sound_data->path;

	if (priority)
		*priority = sound_data->priority;

	return 0;
}
//LCOV_EXCL_STOP

inline int is_sound_mode(void)
{
	return sndstatus;
}

inline int is_touch_sndstatus(void)
{
	return touch_sndstatus;
}

inline int is_keytone_sndstatus(void)
{
	return keytone_sndstatus;
}

//LCOV_EXCL_START Not called Callback
static void feedback_sndstatus_cb(keynode_t *key, void* data)
{
	sndstatus = vconf_keynode_get_bool(key);
}

static void feedback_touch_sndstatus_cb(keynode_t *key, void* data)
{
	touch_sndstatus = vconf_keynode_get_bool(key);
}

static void feedback_keytone_sndstatus_cb(keynode_t *key, void* data)
{
	keytone_sndstatus = vconf_keynode_get_bool(key);
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static void feedback_sound_theme_changed_cb(keynode_t *key, void* data)
{
	int vconf_sound_theme_id = vconf_keynode_get_int(key);

	if (vconf_sound_theme_id <= 0) {
		current_theme_id = 0;
		return;
	}

	current_theme_id = vconf_keynode_get_int(key);
}
//LCOV_EXCL_STOP

static void sound_theme_id_init(void)
{
	int ret;
	unsigned int theme_id = SOUND_THEME_ID_UNINITIALIZED;

	ret = vconf_get_int(VCONFKEY_SETAPPL_ACCESSIBILITY_SOUND_FEEDBACK_THEME, (int *) &theme_id);
	if (ret < 0 || theme_id == SOUND_THEME_ID_UNINITIALIZED)
		sound_thememan_get_default_sound_theme_id(&theme_id);

	current_theme_id = theme_id;
}

static void sound_init(void)
{
	int ret = 0;

	ret = sound_thememan_init();
	if (ret < 0) {
		_E("Failed to initialize sound theme manager(%d)", ret);
		sound_thememan_exit();
	}

	ret = sound_parser_init();
	if (ret < 0) {
		_E("Failed to parsing sound config file(%d)", ret);
		sound_thememan_exit();
		sound_parser_exit();
	}

	sound_theme_id_init();

	/* check sound status */
	if (vconf_get_bool(VCONFKEY_SETAPPL_TOUCH_SOUNDS_BOOL, &touch_sndstatus) < 0)
		_W("VCONFKEY_SETAPPL_TOUCH_SOUNDS_BOOL ==> FAIL!!"); //LCOV_EXCL_LINE

	if (vconf_get_bool(VCONFKEY_SETAPPL_BUTTON_SOUNDS_BOOL, &keytone_sndstatus) < 0)
		_W("VCONFKEY_SETAPPL_BUTTON_SOUNDS_BOOL ==> FAIL!!"); //LCOV_EXCL_LINE

	if (vconf_get_bool(VCONFKEY_SETAPPL_SOUND_STATUS_BOOL, &sndstatus) < 0) {
		_D("fail to get sound status, will work as turning off"); //LCOV_EXCL_LINE
		sndstatus = 0; //LCOV_EXCL_LINE
	}

	/* add watch for status value */
	ret = vconf_notify_key_changed(VCONFKEY_SETAPPL_SOUND_STATUS_BOOL, feedback_sndstatus_cb, NULL);
	if (ret != 0)
		_W("Add watch for VCONFKEY_SETAPPL_SOUND_STATUS_BOOL failed"); //LCOV_EXCL_LINE
	ret = vconf_notify_key_changed(VCONFKEY_SETAPPL_TOUCH_SOUNDS_BOOL, feedback_touch_sndstatus_cb, NULL);
	if (ret != 0)
		_W("Add watch for VCONFKEY_SETAPPL_TOUCH_SOUNDS_BOOL failed"); //LCOV_EXCL_LINE
	ret = vconf_notify_key_changed(VCONFKEY_SETAPPL_BUTTON_SOUNDS_BOOL, feedback_keytone_sndstatus_cb, NULL);
	if (ret != 0)
		_W("Add watch for VCONFKEY_SETAPPL_BUTTON_SOUNDS_BOOL failed"); //LCOV_EXCL_LINE
	ret = vconf_notify_key_changed(VCONFKEY_SETAPPL_ACCESSIBILITY_SOUND_FEEDBACK_THEME, feedback_sound_theme_changed_cb, NULL);
	if (ret != 0)
		_W("Add watch for VCONFKEY_SETAPPL_ACCESSIBILITY_SOUND_FEEDBACK_THEME failed"); //LCOV_EXCL_LINE
}

static void sound_exit(void)
{
	int ret;

	sound_parser_exit();
	sound_thememan_exit();

	/* remove watch */
	ret = vconf_ignore_key_changed(VCONFKEY_SETAPPL_SOUND_STATUS_BOOL, feedback_sndstatus_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_SETAPPL_SOUND_STATUS_BOOL failed"); //LCOV_EXCL_LINE

	ret = vconf_ignore_key_changed(VCONFKEY_SETAPPL_TOUCH_SOUNDS_BOOL, feedback_touch_sndstatus_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_SETAPPL_TOUCH_SOUNDS_BOOL failed"); //LCOV_EXCL_LINE
	ret = vconf_ignore_key_changed(VCONFKEY_SETAPPL_BUTTON_SOUNDS_BOOL, feedback_keytone_sndstatus_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_SETAPPL_BUTTON_SOUNDS_BOOL failed"); //LCOV_EXCL_LINE
	ret = vconf_ignore_key_changed(VCONFKEY_SETAPPL_ACCESSIBILITY_SOUND_FEEDBACK_THEME, feedback_sound_theme_changed_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_SETAPPL_ACCESSIBILITY_SOUND_FEEDBACK_THEME failed"); //LCOV_EXCL_LINE

}

//LCOV_EXCL_START
static void play_completed(int id, void *data)
{
	int ret;
	sound_stream_info_h stream_info = (sound_stream_info_h)data;

	ret = sound_manager_destroy_stream_information(stream_info);
	if (ret != SOUND_MANAGER_ERROR_NONE)
		_E("Failed to set session to play sound(%d)", ret); //LCOV_EXCL_LINE
}
//LCOV_EXCL_STOP

static int sound_play(feedback_pattern_e pattern, bool always)
{
	struct stat buf;
	int retry = FEEDBACK_RETRY_CNT, ret;
	sound_stream_info_h stream_info;
	const char *path;
	int level;

	if (vconf_get_bool(VCONFKEY_SETAPPL_SOUND_STATUS_BOOL, &sndstatus) < 0) {
		_D("fail to get sound status, will work as turning off"); //LCOV_EXCL_LINE
		sndstatus = 0; //LCOV_EXCL_LINE
	}

	if (always) {
		_W("Always on condition(pattern %s)", profile->str_pattern(pattern)); //LCOV_EXCL_LINE
		goto check_pattern;
	}

	if (sndstatus == 0 && profile->get_always_alert_case &&
	    !profile->get_always_alert_case(FEEDBACK_TYPE_SOUND, pattern)) {
		_D("Sound condition is OFF (sndstatus : %d)", sndstatus); //LCOV_EXCL_LINE
		return 0;
	}

	if (sndstatus && profile->get_always_off_case &&
	    profile->get_always_off_case(FEEDBACK_TYPE_SOUND, pattern)) {
		_D("Sound always off condition(pattern %s)", profile->str_pattern(pattern)); //LCOV_EXCL_LINE
		return 0;
	}

check_pattern:
	/* get sound file path */
	path = get_data(pattern);
	if (!path || stat(path, &buf)) {
		//LCOV_EXCL_START
		_E("Not supported sound pattern(pattern %d)", pattern); //LCOV_EXCL_LINE
		return -ENOTSUP;
		//LCOV_EXCL_STOP
	}

	/* play sound file */
	if (profile->is_notification_type && profile->is_notification_type(pattern)) {
		ret = sound_manager_create_stream_information(SOUND_STREAM_TYPE_NOTIFICATION,
                NULL, NULL, &stream_info);
		if (ret != SOUND_MANAGER_ERROR_NONE) {
			//LCOV_EXCL_START
			_E("Failed to set session to play sound(%d)", ret);
			return -ENOENT;
			//LCOV_EXCL_STOP
		}
		do {
			ret = wav_player_start_new(path, stream_info, play_completed, stream_info, NULL);
			if (ret == WAV_PLAYER_ERROR_NONE) {
				_D("Wav Play success! SND filename is %s", path); //LCOV_EXCL_LINE
				return 0;
			}
			_E("Failed to play sound file (%d, %s)", ret, path); //LCOV_EXCL_LINE
		} while (retry--); //LCOV_EXCL_LINE
	} else {
		do {
			if (profile->get_strength_type)
				level = profile->get_strength_type(FEEDBACK_TYPE_SOUND, pattern);
			else
				level = VOLUME_TYPE_SYSTEM;

			ret = mm_sound_play_keysound(path, level);
			if (ret == MM_ERROR_NONE) {
				_D("Play success! %s(%s)", profile->str_pattern(pattern), path);
				return 0;
			}
			_E("mm_sound_play_keysound() returned error(%d)", ret); //LCOV_EXCL_LINE
		} while (retry--); //LCOV_EXCL_LINE
	}
	return -EPERM;
}

//LCOV_EXCL_START
static int sound_play_path(feedback_pattern_e pattern, const char *soundpath, bool always)
{
	struct stat buf;
	int retry = FEEDBACK_RETRY_CNT, ret;
	sound_stream_info_h stream_info;
	int level;

	if (!soundpath || stat(soundpath, &buf)) {
		if (!soundpath) //LCOV_EXCL_LINE
			_E("Not supported sound pattern(pattern %d)", pattern); //LCOV_EXCL_LINE
		else
			_E("Not supported sound pattern(soundpath %s of pattern %s)", soundpath, profile->str_pattern(pattern)); //LCOV_EXCL_LINE
		return -ENOTSUP; //LCOV_EXCL_LINE
	}

	if (vconf_get_bool(VCONFKEY_SETAPPL_SOUND_STATUS_BOOL, &sndstatus) < 0) {
		_D("fail to get sound status, will work as turning off"); //LCOV_EXCL_LINE
		sndstatus = 0; //LCOV_EXCL_LINE
	}

	if (always) {
		_W("Always on condition(pattern %s)", profile->str_pattern(pattern)); //LCOV_EXCL_LINE
		goto check_pattern;
	}

	if (sndstatus == 0 && profile->get_always_alert_case &&
	    !profile->get_always_alert_case(FEEDBACK_TYPE_SOUND, pattern)) {
		_D("Sound condition is OFF (sndstatus : %d)", sndstatus); //LCOV_EXCL_LINE
		return 0;
	}

	if (sndstatus && profile->get_always_off_case &&
	    profile->get_always_off_case(FEEDBACK_TYPE_SOUND, pattern)) {
		_D("Sound always off condition(pattern %s)", profile->str_pattern(pattern)); //LCOV_EXCL_LINE
		return 0;
	}

check_pattern:
	/* play sound file */
	if (profile->is_notification_type && profile->is_notification_type(pattern)) {
		ret = sound_manager_create_stream_information(SOUND_STREAM_TYPE_NOTIFICATION,
                NULL, NULL, &stream_info);
		if (ret != SOUND_MANAGER_ERROR_NONE) {
			_E("Failed to set session to play sound(%d)", ret); //LCOV_EXCL_LINE
			return -ENOENT;
		}
		do {
            ret = wav_player_start_new(soundpath, stream_info, play_completed, stream_info, NULL);
			if (ret == WAV_PLAYER_ERROR_NONE) {
                _D("Wav Play success! SND filename is %s", soundpath); //LCOV_EXCL_LINE
				return 0; //LCOV_EXCL_LINE
			}
            _E("Failed to play sound file (%d, %s)", ret, soundpath); //LCOV_EXCL_LINE
		} while (retry--); //LCOV_EXCL_LINE
	} else {
		do {
			if (profile->get_strength_type)
				level = profile->get_strength_type(FEEDBACK_TYPE_SOUND, pattern);
			else
				level = VOLUME_TYPE_SYSTEM;

			ret = mm_sound_play_keysound(soundpath, level);
			if (ret == MM_ERROR_NONE) {
				_D("Play success! %s(%s)", profile->str_pattern(pattern), soundpath);
				return 0;
			}
			_E("mm_sound_play_keysound(\'%s\') returned error(%d)", soundpath, ret); //LCOV_EXCL_LINE
		} while (retry--); //LCOV_EXCL_LINE
	}
	return -EPERM;
}
//LCOV_EXCL_STOP

static int sound_is_supported(feedback_pattern_e pattern, bool *supported)
{
	struct stat buf;
	const char *path;
	bool ret = true;

	if (!supported) {
		_E("Invalid parameter : supported(NULL)"); //LCOV_EXCL_LINE
		return -EINVAL; //LCOV_EXCL_LINE
	}

	/* get sound file path */
	path = get_data(pattern);
	if (!path || stat(path, &buf)) {
		_D("%d is not presents", pattern);
		ret = false;
	}

	*supported = ret;
	return 0;
}

static int sound_stop(void)
{
	int ret;

	/**
	* @param	filename	[in] keytone filename to stop (can be null if whole exist keytones needs stops)
	* int mm_sound_stop_keysound(const char *filename);
	*/
	ret = mm_sound_stop_keysound(NULL);
	if (ret == MM_ERROR_NONE)	{
		_D("stop success!");
		return 0;
	}
	_E("mm_sound_stop_keysound() returned error(%d)", ret); //LCOV_EXCL_LINE
	return ret; //LCOV_EXCL_LINE
}

//LCOV_EXCL_START Not used function
static int sound_get_path(feedback_pattern_e pattern, char *buf, unsigned int buflen)
{
	const char *cur_path;
	int ret = 0;

	if (!buf || buflen == 0)
		return -EINVAL;

	/* get sound file path */
	cur_path = get_data(pattern);
	if (!cur_path) {
		_E("This pattern(%s) in sound type is not supported to play",
				profile->str_pattern(pattern));
		cur_path = "NULL";
		ret = -ENOENT;
	}

	snprintf(buf, buflen, "%s", cur_path);
	return ret;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int sound_get_count_of_theme(unsigned int *count_of_theme)
{
	int ret = 0;
	if (!count_of_theme)
		return -EINVAL;

	ret = sound_thememan_get_number_of_theme(count_of_theme);
	if (ret < 0)
		return -EPERM;

	return 0;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int sound_get_theme_id(unsigned int *id_of_theme)
{
	if (!id_of_theme)
		return -EINVAL;

	*id_of_theme = current_theme_id;

	return 0;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int sound_set_theme_id(unsigned int id_of_theme)
{
	int ret = 0;

	if (!sound_thememan_is_sound_theme_id_exist(id_of_theme))
		return -EINVAL;

	ret = vconf_set_int(VCONFKEY_SETAPPL_ACCESSIBILITY_SOUND_FEEDBACK_THEME, id_of_theme);
	if (ret < 0) {
		_E("Failed to set vconf value for VCONFKEY_SETAPPL_ACCESSIBILITY_SOUND_FEEDBACK_THEME");
		return -EPERM;
	}

	current_theme_id = id_of_theme;

	return 0;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int sound_get_theme_ids(unsigned int *count_of_theme, unsigned int **theme_ids)
{
	int ret = 0;

	if (!count_of_theme || !theme_ids)
		return -EINVAL;

	ret = sound_thememan_get_sound_theme_ids(count_of_theme, theme_ids);
	if (ret < 0)
		return -EPERM;

	return 0;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int sound_play_with_priority(feedback_pattern_e pattern, bool always)
{
	struct stat buf;
	int retry = FEEDBACK_RETRY_CNT;
	int ret = 0;
	sound_stream_info_h stream_info;
	const char *path = NULL;
	int level;
	int priority = -1;

	if (vconf_get_bool(VCONFKEY_SETAPPL_SOUND_STATUS_BOOL, &sndstatus) < 0) {
		_D("fail to get sound status, will work as turning off"); //LCOV_EXCL_LINE
		sndstatus = 0; //LCOV_EXCL_LINE
	}

	if (always) {
		_W("Always on condition(pattern %s)", profile->str_pattern(pattern)); //LCOV_EXCL_LINE
		goto check_pattern;
	}

	if (sndstatus == 0 && profile->get_always_alert_case && !profile->get_always_alert_case(FEEDBACK_TYPE_SOUND, pattern)) {
		_D("Sound condition is OFF (sndstatus : %d)", sndstatus); //LCOV_EXCL_LINE
		return 0;
	}

	if (sndstatus && profile->get_always_off_case && profile->get_always_off_case(FEEDBACK_TYPE_SOUND, pattern)) {
		_D("Sound always off condition(pattern %s)", profile->str_pattern(pattern)); //LCOV_EXCL_LINE
		return 0;
	}

check_pattern:
	/* get sound file path */
	ret = get_sound_pattern_data(pattern, &path, &priority);
	if (ret < 0 || stat(path, &buf)) {
		_E("Not supported sound pattern(pattern %d)", pattern); //LCOV_EXCL_LINE
		return -ENOTSUP;
	}

	/* play sound file */
	if (profile->is_notification_type && profile->is_notification_type(pattern)) {
		ret = sound_manager_create_stream_information(SOUND_STREAM_TYPE_NOTIFICATION,
                NULL, NULL, &stream_info);
		if (ret != SOUND_MANAGER_ERROR_NONE) {
			_E("Failed to set session to play sound(%d)", ret); //LCOV_EXCL_LINE
			return -ENOENT; //LCOV_EXCL_LINE
		}
		do {
		ret = wav_player_start_new(path, stream_info, play_completed, stream_info, NULL);
			if (ret == WAV_PLAYER_ERROR_NONE) {
				_D("Wav Play success! SND filename is %s", path); //LCOV_EXCL_LINE
				return 0;
			}
			_E("Failed to play sound file (%d, %s)", ret, path); //LCOV_EXCL_LINE
		} while (retry--); //LCOV_EXCL_LINE
	} else {
		do {
			if (profile->get_strength_type)
				level = profile->get_strength_type(FEEDBACK_TYPE_SOUND, pattern);
			else
				level = VOLUME_TYPE_SYSTEM;

			ret = mm_sound_play_keysound_priority(path, level, priority);
			if (ret == MM_ERROR_NONE) {
				_D("Play success! %s(%s) priority %d", profile->str_pattern(pattern), path, priority);
				return 0;
			}
			_E("mm_sound_play_keysound_priority() returned error(%d)", ret); //LCOV_EXCL_LINE
		} while (retry--); //LCOV_EXCL_LINE
	}
	return -EPERM;
}
//LCOV_EXCL_STOP

static const struct device_ops sound_device_ops = {
	.type = FEEDBACK_TYPE_SOUND,
	.name = "Sound",
	.init = sound_init,
	.exit = sound_exit,
	.play = sound_play,
	.play_path = sound_play_path,
	.play_with_priority = sound_play_with_priority,
	.stop = sound_stop,
	.is_supported = sound_is_supported,
	.get_path = sound_get_path,
	.set_path = NULL,
	.get_count_of_theme = sound_get_count_of_theme,
	.get_theme_id = sound_get_theme_id,
	.set_theme_id = sound_set_theme_id,
	.get_theme_ids = sound_get_theme_ids,
};

DEVICE_OPS_REGISTER(&sound_device_ops);
