/*
 * libfeedback
 * Copyright (c) 2012 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include <limits.h>
#include <vconf.h>
#include <sys/stat.h>
#include <system_info.h>
#include <libsyscommon/libgdbus.h>

#include "feedback.h"
#include "profiles.h"
#include "feedback-config.h"
#include "devices.h"
#include "log.h"

#define HAPTIC_DEVICE				0

enum haptic_iteration {
	HAPTIC_ITERATION_ONCE = 1,
	HAPTIC_ITERATION_INFINITE = 256,
};

#define METHOD_OPEN                 "OpenDevice"
#define METHOD_CLOSE                "CloseDevice"
#define METHOD_VIBRATE_PATTERN      "VibratePattern"
#define METHOD_STOP                 "StopDevice"
#define METHOD_IS_SUPPORTED         "IsSupported"
#define METHOD_GET_EFFECT           "GetEffect"

#define DEFAULT_DURATION            100
#define SIP_DURATION                60
#define WHITESPACE                  " \t"

#define VIBRATION_FEATURE              "http://tizen.org/feature/feedback.vibration"

static int vibstatus;
static unsigned int v_handle;
static int current_theme_id = 0;

//LCOV_EXCL_START
static inline char *trim_str(char *s)
{
	char *t;
	/* left trim */
	s += strspn(s, WHITESPACE);

	/* right trim */
	for (t = strchr(s, 0); t > s; t--)
		if (!strchr(WHITESPACE, t[-1]))
			break;
	*t = 0;
	return s;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
inline int is_vibration_mode(void)
{
	return vibstatus;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int haptic_open(void)
{
	int ret_dbus, reply;
	ret_dbus = gdbus_call_sync_with_reply_int(VIBRATOR_BUS_NAME, VIBRATOR_PATH_HAPTIC,
			VIBRATOR_INTERFACE_HAPTIC, METHOD_OPEN,
			g_variant_new("(i)", HAPTIC_DEVICE), &reply);
	if (ret_dbus < 0)
		return ret_dbus;
	return reply;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int haptic_close(unsigned int handle)
{
	int ret_dbus, reply;
	ret_dbus = gdbus_call_sync_with_reply_int(VIBRATOR_BUS_NAME, VIBRATOR_PATH_HAPTIC,
			VIBRATOR_INTERFACE_HAPTIC, METHOD_CLOSE,
			g_variant_new("(u)", handle), &reply);
	if (ret_dbus < 0)
		return ret_dbus;
	return reply;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int haptic_is_supported(const char *pattern)
{
	int ret_dbus, reply;
	ret_dbus = gdbus_call_sync_with_reply_int(VIBRATOR_BUS_NAME, VIBRATOR_PATH_HAPTIC,
			VIBRATOR_INTERFACE_HAPTIC, METHOD_IS_SUPPORTED,
			g_variant_new("(s)", pattern), &reply);
	if (ret_dbus < 0)
		return ret_dbus;
	return reply;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int haptic_vibrate_effect(unsigned int handle,
								const char *pattern,
								int feedback,
								int priority)
{
	int ret_dbus, reply;
	ret_dbus = gdbus_call_sync_with_reply_int(VIBRATOR_BUS_NAME, VIBRATOR_PATH_HAPTIC,
			VIBRATOR_INTERFACE_HAPTIC, METHOD_VIBRATE_PATTERN,
			g_variant_new("(usii)", handle, pattern, feedback, priority), &reply);
	if (ret_dbus < 0)
		return ret_dbus;
	return reply;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int haptic_vibrate_stop(unsigned int handle)
{
	int ret_dbus, reply;
	ret_dbus = gdbus_call_sync_with_reply_int(VIBRATOR_BUS_NAME, VIBRATOR_PATH_HAPTIC,
			VIBRATOR_INTERFACE_HAPTIC, METHOD_STOP,
			g_variant_new("(u)", handle), &reply);
	if (ret_dbus < 0)
		return ret_dbus;
	return reply;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int get_priority(feedback_pattern_e pattern)
{
	if (profile->get_priority)
		return profile->get_priority(pattern);
	return PRIORITY_MIDDLE;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START callback
static void feedback_vibstatus_cb(keynode_t *key, void* data)
{
	vibstatus = vconf_keynode_get_bool(key);
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static void feedback_vibration_theme_changed_cb(keynode_t *key, void* data)
{
	current_theme_id = vconf_keynode_get_int(key);
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static void vibrator_init(void)
{
	int ret_val;
	bool haptic_avail;

	if (vconf_get_int(VCONFKEY_SETAPPL_ACCESSIBILITY_VIBRATION_FEEDBACK_THEME, &current_theme_id) < 0)
		_W("Failed to get VCONFKEY_SETAPPL_ACCESSIBILITY_VIBRATION_FEEDBACK_THEME");

	ret_val = system_info_get_platform_bool(VIBRATION_FEATURE, &haptic_avail);
	if (ret_val < 0) { //LCOV_EXCL_LINE
		v_handle = -ENOTSUP; //LCOV_EXCL_LINE
		return; //LCOV_EXCL_LINE
	} else if (ret_val == 0 && !haptic_avail) {
		v_handle = -ENOTSUP; //LCOV_EXCL_LINE
		return; //LCOV_EXCL_LINE
	}

	/* Vibration Init */
	ret_val = haptic_open();
	if (ret_val == -EACCES || ret_val == -ECOMM || ret_val == -EPERM) {
		_E("haptic_open ==> FAIL!! : %d", ret_val); //LCOV_EXCL_LINE
		v_handle = -EACCES; //LCOV_EXCL_LINE System Error
		return; //LCOV_EXCL_LINE System Error
	}
	if (ret_val < 0) {
		_E("haptic_open ==> FAIL!! : %d", ret_val); //LCOV_EXCL_LINE
		v_handle = -ENOTSUP; //LCOV_EXCL_LINE System Error
		return; //LCOV_EXCL_LINE System Error
	}

	/* Set vibration handle */
	v_handle = (unsigned int)ret_val;

	ret_val = vconf_notify_key_changed(VCONFKEY_SETAPPL_VIBRATION_STATUS_BOOL, feedback_vibstatus_cb, NULL);
	if (ret_val != 0)
		_W("Add watch for VCONFKEY_SETAPPL_VIBRATION_STATUS_BOOL failed"); //LCOV_EXCL_LINE
	ret_val = vconf_notify_key_changed(VCONFKEY_SETAPPL_ACCESSIBILITY_VIBRATION_FEEDBACK_THEME, feedback_vibration_theme_changed_cb, NULL);
	if (ret_val != 0)
		_W("Add watch for VCONFKEY_SETAPPL_ACCESSIBILITY_VIBRATION_FEEDBACK_THEME failed");
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static void vibrator_exit(void)
{
	int ret;

	if ((int)v_handle > 0) {
		ret = haptic_close(v_handle);
		if (ret < 0)
			_E("haptic_close is failed : %d", ret); //LCOV_EXCL_LINE
		v_handle = 0;
	}

	ret = vconf_ignore_key_changed(VCONFKEY_SETAPPL_VIBRATION_STATUS_BOOL, feedback_vibstatus_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_SETAPPL_VIBRATION_STATUS_BOOL failed"); //LCOV_EXCL_LINE
	ret = vconf_ignore_key_changed(VCONFKEY_SETAPPL_ACCESSIBILITY_VIBRATION_FEEDBACK_THEME, feedback_vibration_theme_changed_cb);
	if (ret != 0)
		_W("Remove watch for VCONFKEY_SETAPPL_ACCESSIBILITY_VIBRATION_FEEDBACK_THEME failed");
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int vibrator_play(feedback_pattern_e pattern, bool always)
{
	char *temp;
	char *data;
	int ret;
	int level;
	int converted_pattern = pattern;

	if ((int)v_handle <= 0) {
		if (v_handle == 0) { //LCOV_EXCL_LINE System Error
			_E("Not initialized"); //LCOV_EXCL_LINE
			return -ENOENT; //LCOV_EXCL_LINE System Error
		}
		_E("Not supported vibration"); //LCOV_EXCL_LINE
		return v_handle; //LCOV_EXCL_LINE System Error
	}

	if (vconf_get_bool(VCONFKEY_SETAPPL_VIBRATION_STATUS_BOOL, &vibstatus) < 0) {
		_D("fail to get vibration status, will work as turning off"); //LCOV_EXCL_LINE
		vibstatus = 0; //LCOV_EXCL_LINE
	}

	if (always) {
		_W("Always on condition(pattern %s)", profile->str_pattern(pattern)); //LCOV_EXCL_LINE
		goto check_pattern;
	}

	if (vibstatus == 0 && profile->get_always_alert_case &&
	    !profile->get_always_alert_case(FEEDBACK_TYPE_VIBRATION, pattern))  {
		_D("Vibration condition is OFF (vibstatus : %d)", vibstatus); //LCOV_EXCL_LINE
		return 0;
	}

	if (vibstatus && profile->get_always_off_case &&
	    profile->get_always_off_case(FEEDBACK_TYPE_VIBRATION, pattern)) {
		_D("Vibration always off condition"); //LCOV_EXCL_LINE
		return 0;
	}

check_pattern:
	if (pattern <= FEEDBACK_PATTERN_NONE ||
	    pattern >= profile->max_pattern) {
		_E("Not supported vibration pattern"); //LCOV_EXCL_LINE
		return -ENOTSUP; //LCOV_EXCL_LINE
	}

	if (profile->get_strength_type)
		level = profile->get_strength_type(FEEDBACK_TYPE_VIBRATION, pattern);
	else
		level = FEEDBACK_LEVEL(DEFAULT_VIB_LEVEL);

	if (profile->get_strong_buzz)
		converted_pattern = profile->get_strong_buzz(pattern);

	temp = strdup(profile->str_pattern(converted_pattern));
	if (!temp) {
		_E("Failed to get vibration string pattern"); //LCOV_EXCL_LINE
		return -ENOMEM; //LCOV_EXCL_LINE
	}

	data = trim_str(temp);
	if (!data) {
		free(temp); //LCOV_EXCL_LINE
		_E("Not supported vibration pattern"); //LCOV_EXCL_LINE
		return -ENOTSUP; //LCOV_EXCL_LINE
	}

	_D("pattern : %s", data);

	ret = haptic_vibrate_effect(v_handle, data, level, get_priority(converted_pattern));

	if (ret < 0) {
		_E("fail to play vibration"); //LCOV_EXCL_LINE
		free(temp); //LCOV_EXCL_LINE
		if (ret == -ECOMM || ret == -ENOTSUP) //LCOV_EXCL_LINE
			return ret; //LCOV_EXCL_LINE System Error
		return -EPERM; //LCOV_EXCL_LINE
	}

	_D("Play success! Data is %s", data);
	free(temp);
	return 0;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int vibrator_stop(void)
{
	int ret;

	if ((int)v_handle <= 0) {
		if (v_handle == 0) { //LCOV_EXCL_LINE System Error
			_E("Not initialized"); //LCOV_EXCL_LINE
			return -ENOENT; //LCOV_EXCL_LINE System Error
		}
		_E("Not supported vibration"); //LCOV_EXCL_LINE
		return v_handle; //LCOV_EXCL_LINE System Error
	}

	/* stop haptic device */
	ret = haptic_vibrate_stop(v_handle);
	if (ret < 0) {
		_E("haptic_vibrate_stop is failed"); //LCOV_EXCL_LINE
		if (ret == -ECOMM) //LCOV_EXCL_LINE
			return ret; //LCOV_EXCL_LINE System Error
		return -EPERM; //LCOV_EXCL_LINE
	}

	return 0;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START
static int vibrator_is_supported(int pattern, bool *supported)
{
	char *temp;
	char *data;
	int ret = 0;

	if (!supported) {
		_E("Invalid parameter : supported(NULL)"); //LCOV_EXCL_LINE
		return -EINVAL; //LCOV_EXCL_LINE
	}

	if ((int)v_handle <= 0) {
		if (v_handle == 0) { //LCOV_EXCL_LINE System Error
			_E("Not initialized"); //LCOV_EXCL_LINE
			return -ENOENT; //LCOV_EXCL_LINE System Error
		}
		_E("Not supported vibration"); //LCOV_EXCL_LINE
		*supported = false; //LCOV_EXCL_LINE System Error
		return v_handle; //LCOV_EXCL_LINE
	}

	if (pattern <= FEEDBACK_PATTERN_NONE ||
	    pattern >= profile->max_pattern) {
		_E("Not supported vibration pattern"); //LCOV_EXCL_LINE
		*supported = false; //LCOV_EXCL_LINE
		return 0; //LCOV_EXCL_LINE
	}

	/* get vibration data */
	temp = strdup(profile->str_pattern(pattern));
	if (!temp) {
		_E("Failed to get vibration string pattern"); //LCOV_EXCL_LINE
		return -ENOMEM; //LCOV_EXCL_LINE
	}

	data = trim_str(temp);
	if (!data) {
		free(temp); //LCOV_EXCL_LINE
		*supported = false; //LCOV_EXCL_LINE
		return 0; //LCOV_EXCL_LINE
	}

	ret = haptic_is_supported(data);
	free(temp);
	if (ret < 0) {
		_E("fail to get support information"); //LCOV_EXCL_LINE
		if (ret == -ECOMM) //LCOV_EXCL_LINE
			return ret; //LCOV_EXCL_LINE
		return -EPERM; //LCOV_EXCL_LINE
	}

	_D("is_supported : %d", ret);
	*supported = ret;
	return 0;
}
//LCOV_EXCL_STOP

//LCOV_EXCL_START Not Supported Feature
static int vibrator_get_path(feedback_pattern_e pattern, char *buf, unsigned int buflen)
{
	return 0;
}

static int vibrator_set_path(feedback_pattern_e pattern, char *path)
{
	return 0;
}
//LCOV_EXCL_STOP

static const struct device_ops vibrator_device_ops = {
	.type = FEEDBACK_TYPE_VIBRATION,
	.name = "Vibrator",
	.init = vibrator_init,
	.exit = vibrator_exit,
	.play = vibrator_play,
	.stop = vibrator_stop,
	.is_supported = vibrator_is_supported,
	.get_path = vibrator_get_path,
	.set_path = vibrator_set_path,
	.get_count_of_theme = NULL,
	.set_theme_id = NULL,
	.get_theme_id = NULL,
	.get_theme_ids = NULL,
	.play_with_priority = NULL,
};

DEVICE_OPS_REGISTER(&vibrator_device_ops);
